@@ -7,62 +7,88 @@
 #' @param filter_values A data frame or matrix of the data to be analyzed.
 #' @param intervals An integer specifying the number of intervals.
 #' @param percent_overlap Percentage of overlap between consecutive intervals.
-#' @param num_bins_when_clustering Number of bins to use when clustering.
 #' @param methods Specify the clustering method to be used, e.g., "hclust" or "kmeans".
 #' @param method_params A list of parameters for the clustering method
-#' @return A list containing the Mapper graph components:
-#' \item{adjacency}{The adjacency matrix of the Mapper graph.}
-#' \item{num_vertices}{The number of vertices in the Mapper graph.}
-#' \item{level_of_vertex}{A vector specifying the level of each vertex.}
-#' \item{points_in_vertex}{A list of the indices of the points in each vertex.}
-#' \item{points_in_level_set}{A list of the indices of the points in each level set.}
-#' \item{vertices_in_level_set}{A list of the indices of the vertices in each level set.}
+#' @param num_cores Number of cores to use for parallel computing.
+##' @return A list containing the Mapper graph components:
+#' \describe{
+#'   \item{adjacency}{The adjacency matrix of the Mapper graph.}
+#'   \item{num_vertices}{The number of vertices in the Mapper graph.}
+#'   \item{level_of_vertex}{A vector specifying the level of each vertex.}
+#'   \item{points_in_vertex}{A list of the indices of the points in each vertex.}
+#'   \item{points_in_level_set}{A list of the indices of the points in each level set.}
+#'   \item{vertices_in_level_set}{A list of the indices of the vertices in each level set.}
+#' }
+#'
+#' @importFrom parallel makeCluster stopCluster
+#' @importFrom doParallel registerDoParallel
+#' @import foreach
 #' @export
 MapperAlgo <- function(
     filter_values, # dist_df[,1:col]
     intervals, # rep(2, col)
     percent_overlap, # 50
-    num_bins_when_clustering, # 10
     methods,
-    method_params = list() # params in each clustering method
+    method_params = list(), # params in each clustering method
+    num_cores = 1
 ) {
-
+  
   filter_values <- data.frame(filter_values)
   num_intervals <- rep(intervals, ncol(filter_values)) # rep(2,4) = (2,2,2,2)
   
   num_points <- dim(filter_values)[1] # row
-  num_levelsets <- prod(num_intervals) 
+  num_levelsets <- prod(num_intervals)
   
   # define some vectors of length k = number of columns
   filter_min <- as.vector(sapply(filter_values, min))
   filter_max <- as.vector(sapply(filter_values, max))
   interval_width <- (filter_max - filter_min) / num_intervals
   
-  # initialize variables    
+  # initialize variables
   vertex_index <- 0
   level_of_vertex <- c()
   points_in_vertex <- list()
-  points_in_level_set <- vector("list", num_levelsets) 
+  points_in_level_set <- vector("list", num_levelsets)
   # store the data points owned by each individual interval
   vertices_in_level_set <- vector("list", num_levelsets)
-
+  
+  # Set up parallel computing
+  cl <- makeCluster(num_cores)
+  registerDoParallel(cl)
+  
+  # handlers(global = TRUE)
+  # p <- progressor(num_levelsets)
+  
+  results <- foreach(lsfi = 1:num_levelsets,
+                     .packages = c("cluster"),
+                     .export = c("cover_points", "to_lsmi", "perform_clustering", "cluster_cutoff_at_first_empty_bin")) %dopar% {
+                       
+                       # p()
+                       
+                       points_in_level_set <- cover_points(
+                         lsfi, filter_min, interval_width, percent_overlap, filter_values, num_intervals
+                       )
+                       
+                       clustering_result <- perform_clustering(
+                         points_in_level_set,
+                         filter_values,
+                         methods,
+                         method_params
+                       )
+                       
+                       list(
+                         clustering_result = clustering_result,
+                         points_in_level_set = points_in_level_set
+                       )
+                     }
+  
+  stopCluster(cl)
+  
   # begin loop through all level sets
   for (lsfi in 1:num_levelsets) {
-    if (lsfi %% 100 == 0) {
-      message(paste(lsfi, "/", num_levelsets, " processed"))
-    }
-    # Cover step
-    points_in_level_set[[lsfi]] <- cover_points(
-      lsfi, filter_min, interval_width, percent_overlap, filter_values, num_intervals
-      )
-    # Clustering step
-    clustering_result <- perform_clustering(
-        points_in_level_set[[lsfi]], 
-        filter_values, 
-        num_bins_when_clustering, 
-        methods,
-        method_params
-      )
+
+    clustering_result <- results[[lsfi]]$clustering_result
+    points_in_level_set[[lsfi]] <- results[[lsfi]]$points_in_level_set
     
     num_vertices_in_this_level <- clustering_result$num_vertices
     level_external_indices <- clustering_result$external_indices
@@ -75,12 +101,12 @@ MapperAlgo <- function(
       for (j in 1:num_vertices_in_this_level) {
         vertex_index <- vertex_index + 1
         level_of_vertex[vertex_index] <- lsfi # put the current loop count into the corresponding index vertex
-        # let all points that satisfy the condition "the number of internal clusters of the current lsfi == 
+        # let all points that satisfy the condition "the number of internal clusters of the current lsfi ==
         # the maximum value of the current vertices" be put into points_in_vertex
         points_in_vertex[[vertex_index]] <- level_external_indices[level_internal_indices == j]
       }
     }
-    # note : compute the number of points in each cluster of a single interval, 
+    # note : compute the number of points in each cluster of a single interval,
     # and then loop over the number of intervals
   }
   
@@ -97,4 +123,4 @@ MapperAlgo <- function(
   
   class(mapperoutput) <- "TDAmapper"
   return(mapperoutput)
-}
\ No newline at end of file
+}
